<?php

namespace Wp618\Static;

if ( !defined( 'ABSPATH' ) ) exit;

//
class Ajax{
    //
    private $app;

    //
    public function __construct(){
        $this->app = App::getInstance();

        $this->add_ajax('setup_save', [$this, 'cb_setup_save']);
        $this->add_ajax('app_active', [$this, 'app_active']);

        $action = get_action('add_task_save');
        \add_action('wp_ajax_' . $action, [$this, 'cb_add_task_save']);

        $action = get_action('delete_task');
        \add_action('wp_ajax_' . $action, [$this, 'cb_delete_task']);

        $action = get_action('delete_log');
        \add_action('wp_ajax_' . $action, [$this, 'cb_delete_log']);

        $action = get_action('delete_dir_log');
        \add_action('wp_ajax_' . $action, [$this, 'cb_delete_dir_log']);
    }

    //
    private function add_ajax($name, $cb, $is_all_user = false){
        $action = 'wp_ajax_' . Constant::get_full_action($name);
        add_action($action, $cb);

        if($is_all_user){
            $action = 'wp_ajax_nopriv_' . Constant::get_full_action($name);
            add_action($action, $cb);
        }
    }

    //
    public function cb_delete_log(){
        $this->check_params('delete_log');

        $type = $this->app->request->post('type', 'striptags', '');
        if($type == 'all'){
            $this->app->log_model->delete_all('log');
            $msg = '日志全部删除';
        }

        $rdata['msg'] = $msg;
        \wp_send_json_success( $rdata );
        \wp_die();
    }

    //
    public function cb_delete_task(){
        $this->check_params('delete_task');

        $type = $this->app->request->post('type', 'striptags', '');
        if($type == 'all'){
            $this->app->task_model->delete_all('task');
            $msg = '任务全部删除';
        }else{
            $ids = $this->app->request->post('ids', 'striptags', '');
            $id_arr = explode(',', $ids);
            $id_arr = array_map('intval', $id_arr);

            $count = 0;
            foreach ($id_arr as $id){
                if($id > 0){
                    if($this->app->task_model->delete(['id' => $id])){
                        $count++;
                    }
                }
            }

            $msg = '删除 ' . $count . ' 条记录';
        }

        $rdata['msg'] = $msg;
        \wp_send_json_success( $rdata );
        \wp_die();
    }

    //
    public function cb_delete_dir_log(){
        $this->check_params('delete_dir_log');

        $type = $this->app->request->post('type', 'striptags', '');
        if($type == 'all'){
            $this->app->dir_model->delete_all('task');
            $msg = '任务全部删除';
        }else{
            $ids = $this->app->request->post('ids', 'striptags', '');
            $id_arr = explode(',', $ids);
            $id_arr = array_map('intval', $id_arr);

            $count = 0;
            foreach ($id_arr as $id){
                if($id > 0){
                    if($this->app->dir_model->delete(['id' => $id])){
                        $count++;
                    }
                }
            }

            $msg = '删除 ' . $count . ' 条记录';
        }

        $rdata['msg'] = $msg;
        \wp_send_json_success( $rdata );
        \wp_die();
    }

    //
    public function cb_setup_save(){
        $config = get_app_config();

        $option = $this->app->request->post('option', 'striptags', '');

        if(empty($option) || !isset(Constant::SETUP_FIELDS[$option])){
            $rdata['msg'] = '参数错误';
            \wp_send_json_error( $rdata );
            \wp_die();
        }

        $def_data = $this->get_default_items($option);
        $data = [];

        foreach ($def_data as $name => $item){
            $default = $item[1];
            if(isset($item[2]) && $item[2] == 'checkbox'){
                $default = 0;
            }

            $data[$name] = $this->app->request->post($name, $item[0], $default);
        }

        $config[$option] = $data;

        set_app_config($config);

        $rdata['msg'] = '设置成功';
        \wp_send_json_success( $rdata );
        \wp_die();
    }

    //
    private function get_default_items($option){
        if(isset(Constant::SETUP_FIELDS[$option])){
            return Constant::SETUP_FIELDS[$option];
        }

        return [];
    }

    //
    public function cb_add_task_save(){
        $this->check_params('add_task_save');

        $type = $this->app->request->post('type', 'striptags', '');

        $rdata = ['msg' => 'save error default message [2]'];

        $types = [
            'submit_one_id',
            'submit_id_range',
            'submit_hour',
            'submit_date_range',
            'submit_all',
        ];

        if(!in_array($type, $types, true)){
            $rdata['msg'] = 'type 未允许';
            \wp_send_json_error( $rdata );
            \wp_die();
        }

        $post_count = 0;
        if(in_array($type, ['submit_hour', 'submit_date_range'])){
            if($type == 'submit_hour'){
                $hour = $this->app->request->post('hours', 'absint', 0);
                if($hour < 1){
                    $rdata['msg'] = '小时数错误';
                    \wp_send_json_error( $rdata );
                    \wp_die();
                }

                $date_start = date('Y-m-d H:i:s', time() - 3600 * $hour);
                $date_end = date('Y-m-d 23:59:59');
            }

            if($type == 'submit_date_range'){
                $date_start = $this->app->request->post('date_start', 'striptags', '');
                $date_end = $this->app->request->post('date_end', 'striptags', '');
                if(empty($date_start) || empty($date_end)){
                    $rdata['msg'] = '日期不能为空';
                    \wp_send_json_error( $rdata );
                    \wp_die();
                }

                $start_time = min(strtotime($date_start), strtotime($date_end));
                $end_time = max(strtotime($date_start), strtotime($date_end));

                $date_start = date('Y-m-d 00:00', $start_time);
                $date_end = date('Y-m-d 23:59:59', $end_time);
            }

            $post_count = $this->app->taskManage->get_date_range($date_start, $date_end, $type);

            $rdata['msg'] = '任务添加成功，共需要生成 ' . $post_count . ' 个静态';
            \wp_send_json_success( $rdata );
            \wp_die();
        }

        if($type == 'submit_one_id'){
            $id = $this->app->request->post('one_id', 'absint', 0);

            if($id < 1){
                $rdata['msg'] = 'ID 错误';
                \wp_send_json_error( $rdata );
                \wp_die();
            }

            $post_count = $this->app->taskManage->get_id_list_range($id, $id, $type);
        }
        if($type == 'submit_id_range'){
            $start_id = $this->app->request->post('id_range_start', 'absint', 0);
            $end_id = $this->app->request->post('id_range_end', 'absint', 0);

            if($start_id < 1 && $end_id < 1){
                $rdata['msg'] = 'ID 参数错误';
                \wp_send_json_error( $rdata );
                \wp_die();
            }

            $post_count = $this->app->taskManage->get_id_list_range($start_id, $end_id, $type);
        }
        if($type == 'submit_all'){
            $end_row = $this->app->posts_model->get_one([
                'columns' => 'ID',
                'where' => ['post_status' => 'publish'],
                'order' => 'ID desc %^&*()',
            ]);

            if(empty($end_row)){
                $rdata['msg'] = '没有找到数据';
                \wp_send_json_error( $rdata );
                \wp_die();
            }

            $post_count = $this->app->taskManage->get_id_list_range(0, $end_row['ID'], $type);
        }

        $rdata['msg'] = '任务添加成功，共需要生成 ' . $post_count . ' 个静态';
        \wp_send_json_success( $rdata );
        \wp_die();
    }

    //
    private function check_params($action_suffix){
        $action = $this->app->request->post('action', 'wdash');
        $rdata = ['msg' => 'save error default message [1]'];

        if(empty($action)){
            $rdata['msg'] = 'action 参数错误';
            \wp_send_json_error( $rdata );
            \wp_die();
        }

        if(empty($action) || $action != get_action($action_suffix)){
            $rdata['msg'] = '无对应 action';
            \wp_send_json_error( $rdata );
            \wp_die();
        }

        $nonce = $this->app->request->post('nonce', 'striptags');
        if(!\wp_verify_nonce($nonce, $action)){
            $rdata['msg'] = '错误请求' . $nonce;
            \wp_send_json_error( $rdata );
            \wp_die();
        }
    }

    //
    public function app_active(){
        $auth_key = $this->app->request->post('active_code', 'striptags', '');

        if(empty($auth_key)){
            wp_send_json_error( ['msg' => '请输入正确的激活码'] );
            wp_die();
        }

        $user_ip = isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '';

        $url = 'https://wp618.com/wp-json/plugin-auth/v1/auth';

        $auth_content = [
            'auth_key' => $auth_key,
            'app_name' => $this->app->get_app_item('app_name'),
            'app_version' => $this->app->get_app_item('app_version'),
            'user_ip' => $user_ip,
            'site_url' => site_url(),
            'http_host' => isset($_SERVER['HTTP_HOST']) ? $_SERVER['HTTP_HOST'] : '-',
            'server_name' => isset($_SERVER['SERVER_NAME']) ? $_SERVER['SERVER_NAME'] : '-',
            'wp_ver' => get_bloginfo('version'),
        ];

        $args = [
            'body' => ['auth_content' => base64_encode(json_encode($auth_content))],
            'sslverify' => false
        ];
        $json = wp_remote_post($url, $args);

        if(empty($json) || empty($json['body'])){
            wp_send_json_error( ['msg' => '未授权成功: 136'] );
            wp_die();
        }

        $body_json = json_decode($json['body'], true);
        if(empty($body_json['status'])){
            wp_send_json_error( ['msg' => '未授权成功: 139'] );
            wp_die();
        }
        if($body_json['status'] != 'success'){
            wp_send_json_error( ['msg' => $body_json['message']] );
            wp_die();
        }

        $key = App::getInstance()->get_app_item('app_prefix') . 'actived';
        update_option($key, ['is_actived' => 1]);

        wp_send_json_success( ['msg' => '授权成功，请刷新页面'] );
        wp_die();
    }


}