<?php

namespace Wp618\Static;

if ( !defined( 'ABSPATH' ) ) exit;

//
class HtmlBuilder
{
    //
    private $app;

    //
    public function __construct(){
        $this->app = App::getInstance();
    }

    //
    public function create_by_dir($dir_row){
        $result = $this->app->dir_model->create_html_by_dir_row($dir_row);

        if(empty($result['size'])){
            $result['size'] = 1;
        }

        if(empty($result['is_update'])){
            $data = ['size' => $result['size'], 'status_code' => $result['status_code'], 'unix_timestamp' => time()];

            $this->app->dir_model->update($data, ['id' => $dir_row['id']]);
        }

        return $result;
    }

    //
    public function create_by_post_id($post_id){
        $result = $this->create_html_by_post_id($post_id);

        $this->app->task_model->delete(['object_id' => $post_id]);

        return $result;
    }

    //
    private function create_html_by_post_id($post_id){
        $post = $this->app->posts_model->get_one([
            'where' => [
                'ID' => $post_id,
                'post_status' => 'publish'
            ],
        ]);

        $ret = [
            'success' => false,
            'msg' => '',
            'url' => '',
            'title' => '',
            'size' => 0,
        ];

        if(empty($post)){
            $ret['msg'] = 'empty post by get one';
            return $ret;
        }

        $url = get_permalink($post_id);
        $ret['url'] = $url;

        $this->app->dir_model->write_url_dir_list($url); // write dir list

        $path = $this->app->htmlContent->url_2_path_v2($url);
        if(empty($path)){
            $ret['msg'] = 'empty path';
            return $ret;
        }

        $ret['title'] = $post['post_title'];

        $this->app->htmlContent->unlink_old_file($path);

        $result = $this->app->htmlContent->get_html_result($url);

        $write_byte = 0;
        $status = '-';
        if($result['status_code'] == 200 && $result['size'] > 0){
            $write_byte = $this->write_html($path, $result['body'], $post, $url);
            $status = $write_byte > 0 ? 'success' : 'warning-0';
        }

        $this->app->log_model->insert([
            'status' => $status,
            'title' => '写入single，字节: ' . $write_byte,
            'content' => 'URL: ' . $url,
        ]);

        if($write_byte < 1){
            $ret['msg'] = '未写入字节';
            return $ret;
        }

        $ret['success'] = true;
        $ret['msg'] = '成功';
        $ret['size'] = $write_byte;

        return $ret;
    }

    //
    private function write_html($path, $content, $post, $orig_url){
        if(stripos($content, $post['post_title']) === false){ // 判断是否带有标题
            $ret['msg'] = 'body does not contain post title';
            return 0;
        }

        $pos = strripos($content, '</body>');
        if($pos !== false){
            $comment = '<!-- Create Html ' . date('Y-m-d H:i:s') . ' -->';
            $content = substr_replace($content, $comment, $pos, 0);
        }

        $size = write_file($path, $content);

        $this->app->file_model->write($size, $post['ID'], $orig_url, $post['post_title'], $path);

        return $size;
    }

}