<?php

namespace Wp618\Static;

if ( !defined( 'ABSPATH' ) ) exit;

//
class HtmlContent
{
    //
    private $app;

    const INDEX_FILE_NAME = 'index.html';

    //
    public function __construct(){
        $this->app = App::getInstance();
    }

    //
    public function post_url_check($url){
        $url = $orig_url = str_replace('\\', '/', $url);

        $ret = [
            'success' => false,
            'msg' => 'post url check init',
            'url' => $orig_url,
            'path' => '',
            'has_parent_dir' => false,
            'dir_parent_url' => '',
            'dir_parent_path' => '',
        ];

        if(stripos($url, '?') !== false){
            $ret['msg'] = 'post url has ?: ' . $url;

            return $ret;
        }

        if(substr($url, -1) === '/'){
            $ret['success'] = true;
            $ret['msg'] = 'post dir mode url';
            $ret['path'] = $this->url_to_file_path($url, 1);

            return $ret;
        }

        $ext1 = strtolower(substr($url, -4));
        $ext2 = strtolower(substr($url, -5));
        if($ext1 == '.htm' || $ext2 == '.html'){
            $ret['success'] = true;
            $ret['msg'] = 'post file mode url';
            $ret['path'] = $this->url_to_file_path($url, 2);

            $ret['has_parent_dir'] = true;
            $ret['dir_parent_url'] = dirname($orig_url) . '/';
            $ret['dir_parent_path'] = dirname($ret['path']) . '/' . self::INDEX_FILE_NAME;

            return $ret;
        }

        $file = basename($url);
        if(preg_match('#^[\w\-]+$#i', $file)){
            $ret['success'] = true;
            $ret['msg'] = 'post file not ext mode url';
            $ret['path'] = $this->url_to_file_path($url, 3);

            $ret['has_parent_dir'] = true;
            $ret['dir_parent_url'] = dirname($orig_url) . '/';
            $ret['dir_parent_path'] = dirname(dirname($ret['path'])) . '/' . self::INDEX_FILE_NAME;

            return $ret;
        }

        return $ret;
    }

    //
    public function is_html_path($path){
        $ext = get_file_ext($path, true);

        return in_array($ext, ['.html', '.htm']);
    }

    //
    public function url_to_file_path($url, $type){
        $url = str_replace('\\', '/', $url);
        $url = preg_replace('#https?://#i', '', trim($url));
        $path = strstr($url, '/');

        if($type == 1){
            $path .= self::INDEX_FILE_NAME;
        }
        if($type == 3){
            $path .= '/' . self::INDEX_FILE_NAME;
        }

        return $_SERVER['DOCUMENT_ROOT'] . $path;
    }

    //
    public function get_url_hash($url){
        $url = trim(strtolower($url));

        return md5($url);
    }

    //
    public function get_html_result($url){
        $ret = [
            'msg' => 'init',
            'status_code' => 0,
            'size' => 0,
            'body' => '',
        ];

        // 发送 HTTP GET 请求
        $args = [
            'timeout' => 10,
            'sslverify' => false,
        ];
        $response = wp_remote_get($url, $args);

        // 检查是否成功获取响应
        if ( is_wp_error( $response ) ) {
            $ret['msg'] = 'response error';
            return $ret;
        }

        // 获取响应的状态码
        $status_code = wp_remote_retrieve_response_code( $response );
        $ret['status_code'] = $status_code;

        if($status_code != 200){
            $ret['msg'] = 'error http code: ' . $status_code . ', url: ' . $url;
            return $ret;
        }

        $ret['body'] = wp_remote_retrieve_body( $response );
        $ret['size'] = strlen($ret['body']);

        $content = $ret;
        unset($content['body']);
        $content['url'] = $url;
        $content = json_encode($content);
        $this->app->log_model->insert(['status' => 'get_html_log', 'title' => '抓取内容', 'content' => $content]);

        return $ret;
    }

    //
    public function unlink_old_file($path){
        if(!$this->is_html_path($path)){
            return false;
        }

        if(file_exists($path)){
            unlink($path);

            $this->app->log_model->insert([
                'status' => 'success',
                'title' => '删除旧文件',
                'content' => $path,
            ]);
        }

        $dir = dirname($path);
        $list = [
            $dir
        ];

        return $this->app->dir_model->delete_empty_list($list, 1);
    }

    //
    public function get_url_dir_list_v2($url){
        $url = trim(str_replace('\\', '/', $url));
        $parsed_url = parse_url($url);

        if(!preg_match('#^https?://#i', $url)){
            return [];
        }

        if(!isset($parsed_url['path']) || $parsed_url['path'] == '/'){
            return [];
        }

        $path = trim($parsed_url['path']);
        $path = preg_replace('#//+#', '/', $path);

        $list = [];

        $domain = $parsed_url['scheme'] . '://' . $parsed_url['host'];
        if(isset($parsed_url['port'])){
            $domain .= ':' . $parsed_url['port'];
        }

        $arr = explode('/', $path);
        $base_uri = '/';
        for($i = 1; $i < count($arr); $i++){
            if(strtolower(substr($arr[$i], -5)) == '.html' || strtolower(substr($arr[$i], -4)) == '.htm'){
                continue;
            }

            $base_uri .= $arr[$i] . '/';
            $url = $domain . $base_uri;

            $c_path = $this->url_2_path_v2($url);

            $list[] = [
                'url' => $url,
                'path' => $c_path,
            ];
        }

        return $list;
    }

    //
    public function url_2_path_v2($url){
        $url = trim(str_replace('\\', '/', $url));
        $parsed_url = parse_url($url);

        if(empty($parsed_url['path'])){
            return '';
        }

        $path = trim($parsed_url['path']);
        $path = preg_replace('#//+#', '/', $path);

        $root = $_SERVER['DOCUMENT_ROOT'];

        $name = basename($path);
        if(strtolower(substr($name, -5)) != '.html' && strtolower(substr($name, -4)) != '.htm'){
            $path = rtrim($path, '/') . '/index.html';
        }

        return $root . $path;
    }

}