<?php

namespace Wp618\Static;

if ( !defined( 'ABSPATH' ) ) exit;

// 请求处理
class Request
{
    private $filter = null;
    private $data_get = [];
    private $data_post = [];
    private $data_cookie = [];
    private $data_server = [];

    public function __construct()
    {
        $this->filter = new Filter();

        $this->data_get = $_GET;
        $this->data_post = $_POST;
        $this->data_cookie = $_COOKIE;
        $this->data_server = $_SERVER;

        if(function_exists('wp_magic_quotes')){
            $this->data_get = $this->remove_magic_quotes($this->data_get);
            $this->data_post = $this->remove_magic_quotes($this->data_post);
            $this->data_cookie = $this->remove_magic_quotes($this->data_cookie);
            $this->data_server = $this->remove_magic_quotes($this->data_server);
        }
    }

    //
    public function get($name, $filters = null, $default = null){
	    return $this->get_value('get', $name, $filters, $default);
    }

	//
    public function post($name, $filters = null, $default = null){
	    return $this->get_value('post', $name, $filters, $default);
    }

	//
    public function server($name, $filters = null, $default = null){
	    return $this->get_value('server', $name, $filters, $default);
    }

	//
    public function cookie($name, $filters = null, $default = null){
	    return $this->get_value('cookie', $name, $filters, $default);
    }

    /**
     * @param string $type
     * @param string $name
     * @param mixed $filters 支持单个，多个（字符串表示，用斜杆分开），多个（数组表示）
     * @param mixed $default 未获取到值时，默认设置的值
     * @return mixed|null 如果不接受返回 null，需要设置默认值
     */
    private function get_value($type, $name, $filters = null, $default = null){
        $value = $this->get_request_value($type, $name);

        // 如果不接受返回 null，需要设置默认值
        if(!is_null($value)){
            if(is_array($value)){
                // 数组处理，只允许一维数组
                foreach ($value as &$line){
                    // line 不能传入数组，传入后直接返回 null
                    $line = $this->filter->sanitize($line, $filters);
                }
                unset($line);
            }else{
                $value = $this->filter->sanitize($value, $filters);
            }
        }else{
            // 设置默认值
            if(!is_null($default)){
                $value = $default;
            }
        }

        return $value;
    }

    //
    private function remove_magic_quotes($array){
        foreach ( (array) $array as $k => $v ) {
            if ( is_array( $v ) ) {
                $array[ $k ] = $this->remove_magic_quotes( $v );
            } elseif ( is_string( $v ) ) {
                $array[ $k ] = stripslashes( $v );
            } else {
                continue;
            }
        }

        return $array;
    }

    //
    private function get_request_value($type, $name){
        if($type == 'get'){
            $data = $this->data_get;
        }else if($type == 'post'){
            $data = $this->data_post;
        }else if($type == 'cookie'){
            $data = $this->data_cookie;
        }else if($type == 'server'){
            $data = $this->data_server;
        }

        return isset($data[$name]) ? $data[$name] : null;
    }

}