<?php

namespace Wp618\Static;

if ( !defined( 'ABSPATH' ) ) exit;

//
class Menu
{
    const PAGE_TITLE = '生成静态';
    // https://developer.wordpress.org/resource/dashicons/
    const MENU_ICON = 'dashicons-airplane';

    private $app;

    //
    public function __construct(){
        $this->app = App::getInstance();

        \add_action('admin_menu', [$this, 'set_menu']);
    }

    //
    public function set_menu()
    {
        $parent_slug = strtolower(str_replace('\\', '_', __CLASS__));
        $is_actived = $this->app->get_is_actived();

        $title = $menu_title = self::PAGE_TITLE;
        //
        add_menu_page(
            $title,
            $menu_title,
            'manage_options',
            $parent_slug,
            [$this, 'load_setup'],
            self::MENU_ICON,
            100
        );

        $config = get_app_config();
        $is_use_html = !empty($config['general']['is_use_html']);

        //
        if($is_actived && $is_use_html){
            add_submenu_page(
                $parent_slug,
                '添加任务',
                '添加任务',
                'manage_options',
                $this->get_sub_slug(__LINE__),
                [$this, 'add_task']
            );

            //
            add_submenu_page(
                $parent_slug,
                '任务列表',
                '任务列表',
                'manage_options',
                $this->get_sub_slug(__LINE__),
                [$this, 'load_task']
            );

            //
            add_submenu_page(
                $parent_slug,
                '目录任务',
                '目录任务',
                'manage_options',
                $this->get_sub_slug(__LINE__),
                [$this->app->dir, 'log']
            );

            //
            add_submenu_page(
                $parent_slug,
                '日志查看',
                '日志查看',
                'manage_options',
                $this->get_sub_slug(__LINE__),
                [$this, 'load_log']
            );

            //
            add_submenu_page(
                $parent_slug,
                '文件列表',
                '文件列表',
                'manage_options',
                $this->get_sub_slug(__LINE__),
                [$this, 'file_list']
            );
        }
    }

    //
    private function get_sub_slug($num){
        $slug = strtolower(str_replace('\\', '_', __CLASS__));

        return $slug . '_' . substr(md5($num), -8);
    }

    //
    public function add_task(){
        $action = get_action('add_task_save');
        $nonce = \wp_create_nonce($action);

        $data = [
            'action' => $action,
            'nonce' => $nonce,
            'is_use_html_check' => 1,
        ];

        $this->app->display('add_task', $data);
    }

    //
    public function load_setup(){
        if($this->app->get_is_actived()){
            $option = 'general';
            $data = $this->get_setup_data($option);
            $this->app->display('setup', $data);
        }else{
            $action = get_action('app_active');
            $data = [
                'action' => $action,
                'nonce' => \wp_create_nonce($action),
                'option' => 'none',
            ];

            $this->app->display('active', $data);
        }
    }

    //
    private function get_setup_data($option){
        $config = get_app_config();
        $set = $this->get_render_items($option, $config);

        $action = get_action('setup_save');

        $data = [
            'action' => $action,
            'nonce' => \wp_create_nonce($action),
            'option' => $option,
            'set' => $set,
        ];

        return $data;
    }

    //
    private function get_render_items($option, $config){
        if(isset($config[$option])){
            return $config[$option];
        }

        $list = [];

        if(isset(Constant::SETUP_FIELDS[$option])){
            foreach (Constant::SETUP_FIELDS[$option] as $name => $val){
                $list[$name] = $val[1];
            }
        }

        return $list;
    }

    //
    public function load_task(){
        $option = $this->app->request->get('option', 'striptags', '');

        if($option == 'execute'){
            $type = $this->app->request->get('type', 'striptags', '');
            $id_arr = $this->app->request->get('id_arr', 'absint', 0);

            $limit = 5;
            $has_next = false;

            if($type == 'all'){
                $task_list = $this->app->task_model->get_all([
                    'order' => 'id',
                    'limit' => $limit + 1,
                ]);

                if(count($task_list) > $limit){
                    $has_next = true;
                    array_pop($task_list);
                }
            }else{
                $task_list = [];
                foreach ($id_arr as $id){
                    $item = $this->app->task_model->get_one([
                        'where' => ['id' => $id],
                    ]);

                    if($item){
                        $task_list[] = $item;
                    }
                }
            }

            $list = [];
            $id = 1;
            foreach ($task_list as $item){
                $li = $this->app->htmlBuilder->create_by_post_id($item['object_id']);
                $li['id'] = $id++;
                $li['post_id'] = $item['object_id'];

                $list[] = $li;
            }

            $this->app->display('task_list_execute', [
                'list' => $list,
                'type' => $type,
                'has_next' => $has_next,
            ]);
        }else{
            $this->load_task_list();
        }
    }

    //
    private function load_task_list(){
        $page = $this->app->request->get('page', 'wdash', '');
        $paged = $this->app->request->get('paged', 'absint', 1);
        $paged = max(1, $paged);

        $per_page = 20;
        $total = $this->app->task_model->get_count();

        $paging = new Paging();
        $paging->set_args($total, $per_page);

        $list = $this->app->task_model->get_all([
            'order' => 'id desc',
            'offset' => $per_page * ($paged - 1),
            'limit' => $per_page,
        ]);

        $action = get_action('delete_task');
        $nonce = \wp_create_nonce($action);

        $this->app->display('task_list', [
            'per_page' => $per_page,
            'paged' => $paged,
            'total' => $total,
            'list' => $list,
            'paging' => $paging,
            'action' => $action,
            'nonce' => $nonce,
            'page' => $page,
        ]);
    }

    //
    public function load_log(){
        $paged = $this->app->request->get('paged', 'absint', 1);
        $paged = max(1, $paged);

        $per_page = 20;
        $total = $this->app->log_model->get_count();

        $paging = new Paging();
        $paging->set_args($total, $per_page);

        $list = $this->app->log_model->get_all([
            'order' => 'id desc',
            'offset' => $per_page * ($paged - 1),
            'limit' => $per_page,
        ]);

        $action = get_action('delete_log');
        $nonce = \wp_create_nonce($action);

        $this->app->display('log', [
            'per_page' => $per_page,
            'paged' => $paged,
            'total' => $total,
            'list' => $list,
            'paging' => $paging,
            'action' => $action,
            'nonce' => $nonce,
        ]);
    }

    //
    public function file_list(){
        $option = $this->app->request->get('option', 'striptags', '');
        $action = get_action('file_list_ajax');

        if($option == 'delete'){
            $nonce = $this->app->request->get('nonce', 'striptags');
            if(!\wp_verify_nonce($nonce, $action)){
                wp_die('非正常请求!', '错误页面');
            }

            $type = $this->app->request->get('type', 'striptags', '');
            $id_arr = $this->app->request->get('id_arr', 'absint', 0);

            $has_next = false;
            $limit = 20;

            if($type == 'all'){
                $list = $this->app->file_model->get_all([
                    'order' => 'id',
                    'limit' => $limit + 1,
                ]);

                if(count($list) > $limit){
                    $has_next = true;
                    array_pop($list);
                }
            }else{
                $list = [];
                foreach ($id_arr as $id){
                    $item = $this->app->file_model->get_one([
                        'where' => ['id' => $id],
                    ]);

                    if($item){
                        $list[] = $item;
                    }
                }
            }

            $empty_dir_list = [];
            foreach ($list as &$item){
                $item['msg_arr'] = [];

                if(file_exists($item['path'])){
                    $item['msg_arr'][] = '文件存在';

                    $ext = strtolower(get_file_ext($item['path'], true));
                    if(in_array($ext, ['.htm', '.html'], true)){
                        $e_dir = dirname($item['path']);
                        $empty_dir_list[$e_dir] = $e_dir;

                        if(unlink($item['path'])){
                            $item['msg_arr'][] = '正常文件，已删除';
                        }else{
                            $item['msg_arr'][] = '正常文件，删除失败';
                        }
                    }else{
                        $item['msg_arr'][] = '非 html 文件，不删除';
                    }
                }else{
                    $item['msg_arr'][] = '文件不存在';
                }

                $this->app->file_model->delete(['id' => $item['id']]);
            }
            unset($item);

            // delete empty dir
            $this->app->dir_model->delete_empty_list($empty_dir_list, 1);

            $nonce = \wp_create_nonce($action);

            $this->app->display('file_list_delete', [
                'list' => $list,
                'action' => $action,
                'nonce' => $nonce,
                'type' => $type,
                'has_next' => $has_next,
            ]);
        }else{
            $page = $this->app->request->get('page', 'wdash', '');
            $paged = $this->app->request->get('paged', 'absint', 1);
            $paged = max(1, $paged);

            $per_page = 20;
            $total = $this->app->file_model->get_count();

            $paging = new Paging();
            $paging->set_args($total, $per_page);

            $list = $this->app->file_model->get_all([
                'order' => 'id desc',
                'offset' => $per_page * ($paged - 1),
                'limit' => $per_page,
            ]);

            $nonce = \wp_create_nonce($action);

            $this->app->display('file_list', [
                'page' => $page,
                'per_page' => $per_page,
                'paged' => $paged,
                'total' => $total,
                'list' => $list,
                'paging' => $paging,
                'action' => $action,
                'nonce' => $nonce,
            ]);
        }
    }

}
