<?php

namespace Wp618\Static;

if ( !defined( 'ABSPATH' ) ) exit;

//
class Dir_model extends Model
{
    protected $table = 'dir'; // 会被子类覆盖
    protected $is_dev_table = true;

    //
    public function check_url_dir($check_result){
        $this->app->log_model->insert([
            'status' => 'check_result',
            'title' => 'parent检测结果',
            'content' => json_encode($check_result),
        ]);

        if(empty($check_result['has_parent_dir'])){
            return true;
        }

        $path = $check_result['dir_parent_path'];
        $index = str_ireplace('index.html', 'index.php', $path);
        $index = str_ireplace('index.htm', 'index.php', $index);
        if(file_exists($index)){
            $this->app->log_model->insert([
                'status' => 'root_dir',
                'title' => '根目录',
                'content' => $path,
            ]);

            return true;
        }

        $url = $check_result['dir_parent_url'];
        $url_hash = $this->app->htmlContent->get_url_hash($url);

        $row = $this->get_one([
            'where' => ['url_hash' => $url_hash],
        ]);

        $is_update_html = false;
        if(empty($row)){
            $is_update_html = true;
        }else{
            if($row['update_timestamp'] < (time() - 3600 * 12)){
                $is_update_html = true;
            }
        }

        if(!$is_update_html){
            $this->app->log_model->insert([
                'status' => 'check',
                'title' => '目录检测有效期内',
                'content' => 'URL: ' . $url,
            ]);

            return true;
        }

        $this->app->log_model->insert([
            'status' => 'check',
            'title' => '获取目录内容',
            'content' => 'URL: ' . $url,
        ]);

        $this->app->htmlContent->unlink_old_file($path);

        $result = $this->app->htmlContent->get_html_result($url);

        $data = [
            'url' => $url,
            'size' => $result['size'],
            'status_code' => $result['status_code'],
            'update_timestamp' => time(),
            'sub_url' => $check_result['url'],
            'url_hash' => $url_hash,
            'path' => $path,
        ];

        if(empty($row)){
            $this->insert($data);
        }else{
            $this->update($data, ['id' => $row['id']]);
        }

        if($result['size'] > 0 && $result['status_code'] == 200){
            $size = write_file($path, $result['body']);

            $this->app->file_model->write($size, 0, $url, '目录生成', $path);

            $this->app->log_model->insert([
                'status' => 'success',
                'title' => '写入dir，字节: ' . $size,
                'content' => 'URL: ' . $url,
            ]);
        }

        return $result['status_code'] > 0;
    }

    //
    public function create_html_by_dir_row($dir){
        $ret = [
            'success' => true,
            'msg' => 'dir init',
            'size' => 1,
            'status_code' => 1,
            'is_update' => false,
            'url' => $dir['url'],
        ];

        $path = $dir['path'];
        $index = str_ireplace('index.html', 'index.php', $path);
        $index = str_ireplace('index.htm', 'index.php', $index);
        if(file_exists($index)){
            $this->app->log_model->insert([
                'status' => 'root_dir',
                'title' => '根目录',
                'content' => $path,
            ]);

            $ret['msg'] = '有 index.php 文件, 可能是根目录，不生成';

            return $ret;
        }

        $this->app->log_model->insert([
            'status' => 'check',
            'title' => '获取目录内容',
            'content' => 'URL: ' . $dir['url'],
        ]);

        $this->app->htmlContent->unlink_old_file($path);

        $result = $this->app->htmlContent->get_html_result($dir['url']);

        $data = [
            'size' => max($result['size'], 1),
            'status_code' => $result['status_code'],
            'update_timestamp' => time(),
        ];

        $ret['size'] = $result['size'];
        $ret['status_code'] = $result['status_code'];

        $this->update($data, ['id' => $dir['id']]);

        $size = 0;
        if($result['size'] > 0 && $result['status_code'] == 200){
            $size = write_file($path, $result['body']);

            $this->app->file_model->write($size, 0, $dir['url'], '目录生成', $path);

            $this->app->log_model->insert([
                'status' => 'success',
                'title' => '写入dir，字节: ' . $size,
                'content' => 'URL: ' . $dir['url'],
            ]);
        }

        $ret['msg'] = '写入 ' . $ret['size'] . ' 字节';
        $ret['is_update'] = true;

        return $ret;
    }

    //
    public function delete_empty_list($empty_dir_list, $level_count){
        if(empty($empty_dir_list) || $level_count >= 10){
            return false;
        }

        $list = [];

        foreach ($empty_dir_list as $dir){
            if(!file_exists($dir)){
                continue;
            }

            $contents = scandir($dir);
            $contents = array_diff($contents, ['.', '..']);

            if(empty($contents)){
                if(file_exists($dir)){
                    rmdir($dir);
                }

                $list[] = dirname($dir);
            }
        }

        return $this->delete_empty_list($list, $level_count + 1);
    }

    //
    public function write_url_dir_list($sub_url){
        $list = $this->app->htmlContent->get_url_dir_list_v2($sub_url);

        foreach ($list as $item){
            $url_hash = $this->get_url_v2_hash($item['url']);

            $data = [
                'url' => $item['url'],
                'size' => 0,
                'status_code' => 0,
                'update_timestamp' => time(),
                'sub_url' => $sub_url,
                'url_hash' => $this->get_url_v2_hash($item['url']),
                'path' => $item['path'],
            ];

            $row = $this->get_one([
                'where' => ['url_hash' => $url_hash]
            ]);

            if(empty($row)){
                $this->insert($data);
            }
        }
    }

    //
    private function get_url_v2_hash($url){
        $url = trim(strtolower($url));

        return md5($url);
    }

}